#---------------------------------------------------------------------------------------------------------#
# Title: EJEMPLO DE APLICACION DE METODOLOGIA CRISP-DM EN PROYECTO DE ANALISIS DE SENTIMIENTO             #
# Datos: Recopilacion de publicaciones de Twitter relacionadas as COVID/ QUARANTINE & HOBBIES             #
#                         PASO 04 (EXPLORACION Y MODELADO DATOS)                                          #
#                                                                                                         #
# Autor: Paula Munoz                                                                                      #
#                                                                                                         #
# Fecha: 10/17/2020                                                                                       #
#---------------------------------------------------------------------------------------------------------#


### PASO 4 - CRISP-DM: EXPLORACION Y MODELADO DATOS ----

# * Seleccionar tecnica para modelar y/o explorar datos
# * Generar plan de prueba
# * construir modelos (Si aplica)
# * Testear 

## 4.1 INSTALAR Y CARGAR LIBRERIAS ----

#install.packages("igraph")  # Quitar comentario "#" si necesita instalar libreria(s)


library(tidyverse)
library(DataExplorer)
library(tidytext)
library(igraph)
library(ggraph)
library(wordcloud2)




# * Importar Dataset previamente limpiado y listo para analisis ----

covid_hobbies_data <- read_csv("step02_and_step03_data_understanding_and_prep/Data_cleaning_in_Tableau_Prep_02/covid_hobbies_clean_02_from_prep_to_R.csv", col_types = cols(.default = "c"))

covid_hobbies_data 
## 4.2 SELECCIONAR TECNICA PARa MODELAR Y/O EXPLOARAR DATOS ---
#      Para nuestro caso queremos hacer clasificacion y analisis de sentimiento, para exto necesitamos convertir texto en tokens 


# 4.2.1 CONVERTIR TEXTO EN TOKENS ----
#       Funcion unnest_tokens de libreria tidytext

tweets_tokenized <- covid_hobbies_data %>%
    select(row_id, text_clean) %>%
    unnest_tokens(word,text_clean)


tweets_tokenized %>%
    count(word, sort = T)


# * Elimiar stop_words ----


# Importar dataset stop_words como "the", "of"..
data("stop_words")


# Eliminamos los stop_words haciendo anti_join entre nuestro tweets_tokenized dataframe y el stop_words dataframe

tweets_tokenized_clean <- tweets_tokenized %>%
    anti_join(stop_words)  


tweets_tokenized_clean %>%
    count(word, sort = T)

# GENERANDO UN WORDCLOUD EN R STUDIO

wc_tweets_tokenized_clean <- tweets_tokenized_clean %>%
    filter(word != "hobby") %>%
    filter(word != "hobbies") %>%
    filter(word != "pandemic") %>%
    filter(word != "quarantine") %>%
    filter(word != "covid") %>%
    filter(word != "im") %>%
    filter(word != "ive") %>%
    filter(word != "amp") %>%
    filter(word != "ve") %>%
    count(word, sort = T)


wordcloud2(data=wc_tweets_tokenized_clean, size=1.2, color='random-dark')



# 4.2.2 Exportar Dataframe con Tokens para analizar y visualizar en Tableau los Hobbies/ Actividades comunes ----

tweets_tokenized_clean %>% ## Used for Hobbies in Tableau
    write_csv("Data_analysis_files/01_covid_hobbies_activities.csv")



# 4.3 SEPARAR TEXTO EN BI-GRAMS (Dos Palabras consecutivas) ----

tweets_tokenized_bigrams <- covid_hobbies_data %>%
    select(row_id, text_clean) %>%
    unnest_tokens(bigram, text_clean, token = "ngrams", n = 2)


tweets_tokenized_bigrams %>%
    count(bigram, sort = T)


# 4.3.1 Exportar Dataframe con Bi-Grams para analizar y visualizar en Tableau  ----

tweets_tokenized_bigrams %>% ## Use for bigrams in Tableau
    write_csv("Data_analysis_files/02_covid_hobbies_bigrams.csv")



# 4.4 ANALISIS DE SENTIMIENTO ----

# 4.4.1 Importar Diccionario de sentimientos "NRC" ----

nrc_dict <- get_sentiments(lexicon = "nrc")


# Hay 10 Tipos de Sentimientos/Emociones representadas en el diccionario NRC
nrc_dict %>%
    count(sentiment, sort = T) %>% View()


# 4.4.2 Integramos nuestro tokenized dataframe con el dataframe del diccionario nrc ----

tweets_nrc <-  tweets_tokenized_clean %>%
    inner_join(nrc_dict) 


tweets_nrc %>%
    count(sentiment, sort = T)

# Bar chart
g <- tweets_nrc %>%
    count(sentiment, sort = T) 
  
g %>%
    ggplot(aes(sentiment, n, fill = sentiment)) +
    geom_col() +
    theme_classic()



# 4.4.3 Crear una variable nueva para capturar sentimiento general ( Positivo vs Negativo) ----

tweets_nrc_overall_sentiment <- tweets_nrc %>%
    mutate(overall_sentiment = case_when(
        sentiment == 'positive' ~ 'positive',
        sentiment == 'joy' ~ 'positive',
        sentiment == 'trust' ~ 'positive',
        sentiment == 'surprise' ~ 'positive',
        sentiment == 'anticipation' ~ 'positive',
        sentiment == 'negative' ~ 'negative',
        sentiment == 'fear' ~ 'negative',
        sentiment == 'sadness' ~ 'negative',
        sentiment == 'anger' ~ 'negative',
        sentiment == 'disgust' ~ 'negative'
    ))

tweets_nrc_overall_sentiment

# 4.4.4 Exportar Dataframe con sentimientos para analizar y visualizar en Tableau  ----

tweets_nrc_overall_sentiment %>% ## Use for Sankey in Tableau
    write_csv("Data_analysis_files/03_covid_hobbies_nrc_sentiment.csv")



# 4.5   PUNTAJE DE POLARIDAD (Polarity Score) ----

# 4.5.1 Contar # de palabras positivas vs palabras negativas por row_id ----
covid_hobbies_polarity <- tweets_nrc_overall_sentiment %>%
    count(row_id, overall_sentiment) %>%
    
    # Hacer un Pivot horizontal 
    pivot_wider(names_from = overall_sentiment, values_from = n, values_fill = 0) %>%
    
    # Calcular el polarity score por row_id = positive - negative
    mutate(polarity_score = positive - negative ) %>%
    
    # Unir con covid_hobbies_data para traer el dataframe con todo el data
    left_join(covid_hobbies_data) 


covid_hobbies_polarity 

# 4.5.2 Exportar Dataframe con el puntaje de polaridad y otras variables para analizar y visualizar en Tableau  ----
covid_hobbies_polarity %>%  ## Use in Tableau for Polarity Score, hashtags, device info
    write_csv("Data_analysis_files/04_covid_hobbies_polarity.csv")



# 4.6 EXTRA - Bi-grams Diagrama de Network ----


bigrams_separated <- tweets_tokenized_bigrams %>%
    separate(bigram, c("word1", "word2"), sep = " ")

bigrams_filtered <- bigrams_separated %>%
    filter(!word1 %in% stop_words$word) %>%
    filter(!word2 %in% stop_words$word)

bigram_counts <- bigrams_filtered %>% 
    count(word1, word2, sort = TRUE)


bigram_counts


# 4.6.1 Network graph en negro

bigram_graph <- bigram_counts %>%
    filter(n > 7) %>%
    graph_from_data_frame()


bigram_graph


ggraph(bigram_graph, layout = "fr") +
    geom_edge_link() +
    geom_node_point() +
    geom_node_text(aes(label = name), vjust = 1, hjust = 1)


a <- grid::arrow(type = "closed", length = unit(.15, "inches"))


# 4.6.2 Network graph en colores

ggraph(bigram_graph, layout = "fr") +
    geom_edge_link(aes(edge_alpha = n), show.legend = FALSE,
                   arrow = a, end_cap = circle(.07, 'inches')) +
    geom_node_point(color = "lightblue", size = 5) +
    geom_node_text(aes(label = name), vjust = 1, hjust = 1, size = 4) +
    theme_void()
